@@ -11,6 +11,16 @@ from bokeh.models.widgets import DataTable, TableColumn
 from bokeh.resources import INLINE
 from jinja2 import Template
 import markdown
+import pkg_resources
+
+JS_RESOURCES = [
+    'simple-datatables_latest.js', 'jquery-3.6.0.slim.min.js',
+    'jquery.dataTables.min.js']
+CSS_RESOURCES = [
+    'bootstrap.min.css', 'simple-datatables_latest.css',
+    'font-awesome_5.15.4_min.css', 'jquery.dataTables.min.css']
+FILT_TABLE_RESOURCES = [
+    'filterable_table_template.css', 'filterable_table_template.js']
 
 
 def _maybe_new_report(section, require_keys=False):
@@ -81,6 +91,21 @@ class HTMLSection(OrderedDict):
                       paging=paging, sortable=sortable, **kwargs)
         self._add_item(table, key=key)
 
+    def filterable_table(self, data_frame, index=False, key=None,
+                         table_params=None):
+        """Add a filterable table report component.
+
+        :param data_frame: pandas dataframe instance.
+        :param index: include dataframe index in output.
+        :param key: unique key for item.
+        :param table_params: dict of params for DataTables
+            e.g.:  the following sets column 2 and 4 to 5% width,
+            columnDefs: [ { "width": "5%", "targets": [2, 4] }]
+        """
+        table = FilterableTable(data_frame, index=index,
+                                table_params=table_params)
+        self._add_item(table.div, key=key)
+
     def markdown(self, text, key=None):
         """Add markdown formatted text to the report.
 
@@ -200,30 +225,12 @@ class HTMLReport(HTMLSection):
         self.lead = lead
         self.sections = OrderedDict()
         self.sections['main'] = self
-        self.template = Template(
-            """\
-            <!doctype html>
-            <html lang="en">
-            <head>
-                <meta charset="utf-8">
-                <meta http-equiv="X-UA-Compatible" content="IE=edge">
-                <meta name="viewport" content="width=device-width, initial-scale=1">
-                    <title>{{ title }}</title>
-<link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/3.4.1/css/bootstrap.min.css" integrity="sha384-HSMxcRTRxnN+Bdg0JdbxYKrThecOKuH5zCYotlSAcp1+c8xmyTe9GYg1l9a69psu" crossorigin="anonymous">
-<link href="https://cdn.jsdelivr.net/npm/simple-datatables@latest/dist/style.css" rel="stylesheet" type="text/css">
-<script src="https://cdn.jsdelivr.net/npm/simple-datatables@latest" type="text/javascript"></script>
-                    {{ resources }}
-                    {{ script }}
-                </head>
-                <body>
-            <div class="container">
-              <h1>{{ title }}</h1>
-              <p class="lead">{{ lead }}
-            {{ div }}
-                </body>
-            </html>
-            """  # noqa
-        )
+
+        template = pkg_resources.resource_filename(
+            __package__, 'data/report_template.html')
+        with open(template, 'r') as fh:
+            template = fh.read()
+        self.template = Template(template)
 
     def add_section(self, key=None, section=None, require_keys=False):
         """Add a section (grouping of items) to the report.
@@ -242,7 +249,17 @@ class HTMLReport(HTMLSection):
 
     def render(self):
         """Generate HTML report containing figures."""
-        resources = INLINE.render()
+        bokeh_resources = INLINE.render()
+
+        libs = []
+        for resources, stub in (
+                [CSS_RESOURCES, "<style>{}</style>"],
+                [JS_RESOURCES, '<script type="text/javascript">{}</script>']):
+            for res in resources:
+                fn = pkg_resources.resource_filename(
+                    __package__, 'data/{}'.format(res))
+                with open(fn) as fh:
+                    libs.append(stub.format(fh.read()))
 
         all_scripts = list()
         all_divs = list()
@@ -255,7 +272,8 @@ class HTMLReport(HTMLSection):
         divs = '\n'.join(all_divs)
         return self.template.render(
             title=self.title, lead=self.lead,
-            resources=resources, script=script, div=divs)
+            bokeh_resources=bokeh_resources, resources="\n".join(libs),
+            script=script, div=divs)
 
     def write(self, path):
         """Write html report to file."""
@@ -373,9 +391,45 @@ class Table():
         for key, val in kwargs.items():
             if isinstance(val, bool):
                 kwargs[key] = str(val).lower()
-        key = str(uuid.uuid4())
-        self.div = template.render(dataframe=data_frame.to_html(
-                                   table_id=str('a'+key[0:5]),
-                                   index=index),
-                                   table_id=str('a'+key[0:5]),
-                                   kwargs=kwargs)
+        key = str(uuid.uuid4()).replace('-', '')
+        key = "a{}".format(key[1:])
+        # html id must not begin with numeric
+
+        self.div = template.render(
+            dataframe=data_frame.to_html(table_id=key, index=index),
+            table_id=key, kwargs=kwargs)
+
+
+class FilterableTable:
+    """A table report component."""
+
+    def __init__(self, data_frame, index=False, table_params=None):
+        """Initialize table component.
+
+        :param dataframe: dataframe to turn in to filterable table.
+        :param index: whether to include index or not
+        :table params: dict of parameters for DataTables
+        """
+        template = Template(self._get_template())
+
+        key = str(uuid.uuid4()).replace('-', '')
+        key = "a{}".format(key[1:])
+
+        if table_params is None:
+            tp = ''
+        else:
+            tp = str(table_params).strip('{}')
+            tp = f"{tp},"
+        self.div = template.render(
+            dataframe=data_frame.to_html(table_id=key, index=index),
+            table_id=key, datatables_params=tp)
+
+    def _get_template(self):
+        template = ""
+        for res in FILT_TABLE_RESOURCES:
+            fn = pkg_resources.resource_filename(
+                __package__, 'data/{}'.format(res))
+            with open(fn, 'r') as fh:
+                temp_part = fh.read()
+                template += temp_part
+        return template
